@@ -1,6 +1,5 @@
-from __future__ import absolute_import
 import logging
-from contextlib import contextmanager, closing
+from contextlib import contextmanager
 
 import amqp
 
@@ -9,7 +8,7 @@ from kuyruk.task import Task
 from kuyruk.config import Config
 from kuyruk.worker import Worker
 
-__version__ = '5.1.2'
+__version__ = '7.0.1'
 __all__ = ['Kuyruk', 'Config', 'Task', 'Worker']
 
 
@@ -47,7 +46,7 @@ class Kuyruk(object):
         self.config = config
         self.extensions = {}
 
-    def task(self, queue='kuyruk', local=False, retry=0, max_run_time=None):
+    def task(self, queue='kuyruk', retry=0, max_run_time=None):
         """
         Wrap functions with this decorator to convert them to *tasks*.
         After wrapping, calling the function will send a message to
@@ -58,8 +57,6 @@ class Kuyruk(object):
             The failed task will be retried in the same worker.
         :param max_run_time: Maximum allowed time in seconds for task to
             complete.
-        :param local: Append hostname to the queue name. Worker needs to be
-            started on the local queue to receive this task.
         :return: Callable :class:`~kuyruk.Task` object wrapping the original
             function.
 
@@ -70,7 +67,7 @@ class Kuyruk(object):
                 queue_ = 'kuyruk' if callable(queue) else queue
 
                 return Task(
-                    f, self, queue=queue_, local=local,
+                    f, self, queue=queue_,
                     retry=retry, max_run_time=max_run_time)
             return inner
 
@@ -88,8 +85,14 @@ class Kuyruk(object):
         with self.connection() as conn:
             ch = conn.channel()
             logger.info('Opened new channel')
-            with closing(ch):
+            try:
                 yield ch
+            finally:
+                if ch.is_open:
+                    try:
+                        ch.close()
+                    except IOError:
+                        pass
 
     @contextmanager
     def connection(self):
@@ -107,5 +110,17 @@ class Kuyruk(object):
             pass
 
         logger.info('Connected to RabbitMQ')
-        with closing(conn):
+        try:
             yield conn
+        finally:
+            if _is_alive(conn):
+                conn.close()
+
+
+def _is_alive(conn):
+    try:
+        conn.send_heartbeat()
+    except IOError:
+        return False
+    else:
+        return True
