@@ -1,11 +1,9 @@
-from __future__ import absolute_import
 import os
 import sys
 import json
 import signal
 import socket
 import logging
-import traceback
 from uuid import uuid1
 from datetime import datetime
 from contextlib import contextmanager
@@ -24,12 +22,11 @@ class Task(object):
     and sends it to the queue.
 
     """
-    def __init__(self, f, kuyruk, queue='kuyruk', local=False,
+    def __init__(self, f, kuyruk, queue='kuyruk',
                  retry=0, max_run_time=None):
         self.f = f
         self.kuyruk = kuyruk
         self.queue = queue
-        self.local = local
         self.retry = retry
         self.max_run_time = max_run_time
         self._send_signal(signals.task_init)
@@ -47,7 +44,8 @@ class Task(object):
         logger.debug("Task.__call__ args=%r, kwargs=%r", args, kwargs)
         self.send_to_queue(args, kwargs)
 
-    def send_to_queue(self, args=(), kwargs={}, host=None, local=False):
+    def send_to_queue(self, args=(), kwargs={},
+                      host=None, wait_result=None, message_ttl=None):
         """
         Sends a message to the queue.
         A worker will run the task's function when it receives the message.
@@ -56,75 +54,62 @@ class Task(object):
         :param kwargs: Keyword arguments that will be passed to task
             on execution.
         :param host: Send this task to specific host. ``host`` will be
-            appended to the queue name.
-        :param local: Send this task to this host. Hostname of current host
-            will be appended to the queue name.
-        :return: :const:`None`
+            appended to the queue name. If ``host`` is "localhost", hostname
+            of the server will be appended to the queue name.
+        :param wait_result:
+            Wait for result from worker for ``wait_result`` seconds.
+            If timeout occurs,
+            :class:`~kuyruk.exceptions.ResultTimeout` is raised.
+            If excecption occurs in worker,
+            :class:`~kuyruk.exceptions.RemoteException` is raised.
+        :param message_ttl:
+            If set, message will be destroyed in queue after ``message_ttl``
+            seconds.
+        :return: Result from worker if ``wait_result`` is set,
+            else :const:`None`.
 
         """
         if self.kuyruk.config.EAGER:
             # Run the task in current process
-            self.apply(*args, **kwargs)
-            return
+            result = self.apply(*args, **kwargs)
+            return result if wait_result else None
 
         logger.debug("Task.send_to_queue args=%r, kwargs=%r", args, kwargs)
-        local = local or self.local
-        queue = get_queue_name(self.queue, host=host, local=local)
+        queue = self._queue_for_host(host)
         description = self._get_description(args, kwargs, queue)
         self._send_signal(signals.task_presend, args=args, kwargs=kwargs,
                           description=description)
+
         body = json.dumps(description)
         msg = amqp.Message(body=body)
+        if wait_result:
+            # Use direct reply-to feature from RabbitMQ:
+            # https://www.rabbitmq.com/direct-reply-to.html
+            msg.properties['reply_to'] = 'amq.rabbitmq.reply-to'
+
+        if message_ttl:
+            msg.properties['expiration'] = str(int(message_ttl * 1000))
+
         with self.kuyruk.channel() as ch:
+            if wait_result:
+                result = Result(ch.connection)
+                ch.basic_consume(queue='amq.rabbitmq.reply-to', no_ack=True,
+                                 callback=result.process_message)
+
             ch.queue_declare(queue=queue, durable=True, auto_delete=False)
             ch.basic_publish(msg, exchange="", routing_key=queue)
-        self._send_signal(signals.task_postsend, args=args, kwargs=kwargs,
-                          description=description)
+            self._send_signal(signals.task_postsend, args=args, kwargs=kwargs,
+                              description=description)
 
-    @contextmanager
-    def run_in_queue(self, args=(), kwargs={}, host=None, local=False):
-        """
-        Returns a context manager to send a message to the queue and
-        getting result back.
+            if wait_result:
+                return result.wait(wait_result)
 
-        :param args: Arguments that will be passed to task on execution.
-        :param kwargs: Keyword arguments that will be passed to task
-            on execution.
-        :param host: Send this task to specific host. ``host`` will be
-            appended to the queue name.
-        :param local: Send this task to this host. Hostname of current host
-            will be appended to the queue name.
-        :return: :const:`None`
-
-        """
-        if self.kuyruk.config.EAGER:
-            result = Result(None)
-            result.result = self.apply(*args, **kwargs)
-            result.exception = None
-            yield result
-
-        logger.debug("Task.run_in_queue args=%r, kwargs=%r", args, kwargs)
-        local = local or self.local
-        queue = get_queue_name(self.queue, host=host, local=local)
-        description = self._get_description(args, kwargs, queue)
-        self._send_signal(signals.task_presend, args=args, kwargs=kwargs,
-                          description=description)
-        body = json.dumps(description)
-        msg = amqp.Message(body=body, reply_to='amq.rabbitmq.reply-to')
-        with self.kuyruk.channel() as ch:
-            result = Result(ch.connection)
-            ch.basic_consume(queue='amq.rabbitmq.reply-to', no_ack=True,
-                             callback=result._process_message)
-            ch.queue_declare(queue=queue, durable=True, auto_delete=False)
-            ch.basic_publish(msg, exchange="", routing_key=queue)
-            self._send_signal(signals.task_postsend, args=args,
-                              kwargs=kwargs, description=description)
-
-            result._start_heartbeat()
-            try:
-                yield result
-            finally:
-                result._stop_heartbeat()
+    def _queue_for_host(self, host):
+        if not host:
+            return self.queue
+        if host == 'localhost':
+            host = socket.gethostname()
+        return "%s.%s" % (self.queue, host)
 
     def _get_description(self, args, kwargs, queue):
         """Return the dictionary to be sent to the queue."""
@@ -169,7 +154,6 @@ class Task(object):
                     with time_limit(self.max_run_time or 0):
                         return self.f(*args, **kwargs)
                 except Exception:
-                    traceback.print_exc()
                     send_signal(signals.task_error, exc_info=sys.exc_info())
                     if tries <= 0:
                         raise
@@ -212,11 +196,3 @@ def time_limit(seconds):
         yield
     finally:
         signal.alarm(0)
-
-
-def get_queue_name(name, host=None, local=False):
-    if host:
-        return "%s.%s" % (name, host)
-    if local:
-        return "%s.%s" % (name, socket.gethostname())
-    return name
