@@ -1,4 +1,3 @@
-from __future__ import absolute_import, print_function
 import os
 import sys
 import json
@@ -8,23 +7,22 @@ import socket
 import signal
 import logging
 import logging.config
-import warnings
 import threading
 import traceback
 import multiprocessing
 
 import amqp
+from monotonic import monotonic
 
 from kuyruk import importer, signals
-from kuyruk.task import get_queue_name
 from kuyruk.heartbeat import Heartbeat
-from kuyruk.exceptions import Reject, Discard, ConnectionError
+from kuyruk.exceptions import Reject, Discard, HeartbeatError
 
 logger = logging.getLogger(__name__)
 
 
 class Worker(object):
-    """Consumes tasks from a queue and runs them.
+    """Consumes tasks from queues and runs them.
 
     :param app: An instance of :class:`~kuyruk.Kuyruk`
     :param args: Command line arguments
@@ -34,34 +32,48 @@ class Worker(object):
         self.kuyruk = app
 
         if not args.queues:
-            raise ValueError("no queue given")
+            args.queues = ['kuyruk']
 
-        self.queues = [get_queue_name(q, local=args.local) for q in args.queues]
+        def add_host(queue):
+            if queue.endswith('.localhost'):
+                queue = queue.rsplit('.localhost')[0]
+                return "%s.%s" % (queue, self._hostname)
+            else:
+                return queue
+
+        self._hostname = socket.gethostname()
+        self.queues = [add_host(q) for q in args.queues]
         self.shutdown_pending = threading.Event()
-        self._pause = False
-        self._started = None
         self.consuming = False
         self.current_task = None
         self.current_args = None
         self.current_kwargs = None
-        self._heartbeat_exc_info = None
-        if self.config.WORKER_MAX_LOAD is None:
-            self.config.WORKER_MAX_LOAD = multiprocessing.cpu_count()
 
+        self._started_at = None
         self._pid = os.getpid()
-        self._hostname = socket.gethostname()
+
+        self._logging_level = app.config.WORKER_LOGGING_LEVEL
+        if args.logging_level is not None:
+            self._logging_level = args.logging_level
+
+        self._max_run_time = app.config.WORKER_MAX_RUN_TIME
+        if args.max_run_time is not None:
+            self._max_run_time = args.max_run_time
+
+        self._max_load = app.config.WORKER_MAX_LOAD
+        if args.max_load is not None:
+            self._max_load = args.max_load
+        if self._max_load == -1:
+            self._max_load == multiprocessing.cpu_count()
+
+        self._threads = []
+        if self._max_load:
+            self._threads.append(threading.Thread(target=self._watch_load))
+        if self._max_run_time:
+            self._threads.append(threading.Thread(target=self._shutdown_timer))
 
         signals.worker_init.send(self.kuyruk, worker=self)
 
-    @property
-    def queue(self):
-        warnings.warn("Worker.queue is deprecated. Use Worker.queues instead.")
-        return self.queues[0]
-
-    @property
-    def config(self):
-        return self.kuyruk.config
-
     def run(self):
         """Runs the worker and consumes messages from RabbitMQ.
         Returns only after `shutdown()` is called.
@@ -74,7 +86,11 @@ class Worker(object):
         from setproctitle import setproctitle
         setproctitle("kuyruk: worker on %s" % ','.join(self.queues))
 
-        self._setup_logging()
+        if self._logging_level:
+            logging.basicConfig(
+                level=getattr(logging, self._logging_level.upper()),
+                format="%(levelname).1s "
+                       "%(name)s.%(funcName)s:%(lineno)d - %(message)s")
 
         signal.signal(signal.SIGINT, self._handle_sigint)
         signal.signal(signal.SIGTERM, self._handle_sigterm)
@@ -82,28 +98,21 @@ class Worker(object):
         signal.signal(signal.SIGUSR1, self._handle_sigusr1)
         signal.signal(signal.SIGUSR2, self._handle_sigusr2)
 
-        self._started = os.times()[4]
+        self._started_at = os.times()[4]
 
-        for f in (self._watch_load, self._shutdown_timer):
-            t = threading.Thread(target=f)
-            t.daemon = True
+        for t in self._threads:
             t.start()
-
-        signals.worker_start.send(self.kuyruk, worker=self)
-        self._consume_messages()
-        signals.worker_shutdown.send(self.kuyruk, worker=self)
+        try:
+            signals.worker_start.send(self.kuyruk, worker=self)
+            self._consume_messages()
+            signals.worker_shutdown.send(self.kuyruk, worker=self)
+        finally:
+            self.shutdown_pending.set()
+            for t in self._threads:
+                t.join()
 
         logger.debug("End run worker")
 
-    def _setup_logging(self):
-        if self.config.WORKER_LOGGING_CONFIG:
-            logging.config.fileConfig(self.config.WORKER_LOGGING_CONFIG)
-        else:
-            level = getattr(logging, self.config.WORKER_LOGGING_LEVEL.upper())
-            fmt = "%(levelname).1s " \
-                  "%(name)s.%(funcName)s:%(lineno)d - %(message)s"
-            logging.basicConfig(level=level, format=fmt)
-
     def _consume_messages(self):
         with self.kuyruk.channel() as ch:
             # Set prefetch count to 1. If we don't set this, RabbitMQ keeps
@@ -111,19 +120,12 @@ class Worker(object):
             ch.basic_qos(0, 1, True)
 
             self._declare_queues(ch)
-            while not self.shutdown_pending.is_set():
-                # Consume or pause
-                if self._pause and self.consuming:
-                    self._cancel_queues(ch)
-                    logger.info('Consumer cancelled')
-                    self.consuming = False
-                elif not self._pause and not self.consuming:
-                    self._consume_queues(ch)
-                    logger.info('Consumer started')
-                    self.consuming = True
+            self._consume_queues(ch)
+            logger.info('Consumer started')
 
-                if self._heartbeat_exc_info:
-                    break
+            while not self.shutdown_pending.is_set():
+                if self._max_load:
+                    self._pause_or_resume(ch)
 
                 try:
                     ch.connection.heartbeat_tick()
@@ -144,7 +146,27 @@ class Worker(object):
             ch.queue_declare(
                 queue=queue, durable=True, auto_delete=False)
 
+    def _pause_or_resume(self, channel):
+        try:
+            load = self._current_load
+        except AttributeError:
+            should_pause = False
+        else:
+            should_pause = load > self._max_load
+
+        if should_pause and self.consuming:
+            logger.warning(
+                'Load is above the treshold (%.2f/%s), '
+                'pausing consumer', load, self._max_load)
+            self._cancel_queues(channel)
+        elif not should_pause and not self.consuming:
+            logger.warning(
+                'Load is below the treshold (%.2f/%s), '
+                'resuming consumer', load, self._max_load)
+            self._consume_queues(channel)
+
     def _consume_queues(self, ch):
+        self.consuming = True
         for queue in self.queues:
             logger.debug("basic_consume: %s", queue)
             ch.basic_consume(queue=queue,
@@ -152,6 +174,7 @@ class Worker(object):
                              callback=self._process_message)
 
     def _cancel_queues(self, ch):
+        self.consuming = False
         for queue in self.queues:
             logger.debug("basic_cancel: %s", queue)
             ch.basic_cancel(self._consumer_tag(queue))
@@ -199,6 +222,14 @@ class Worker(object):
             if reply_to:
                 exc_info = sys.exc_info()
                 self._send_reply(reply_to, message.channel, None, exc_info)
+        except HeartbeatError as e:
+            logger.error('Error while sending heartbeat')
+            exc_info = e.exc_info
+            logger.error(''.join(traceback.format_exception(*exc_info)))
+            signals.worker_failure.send(self.kuyruk, description=description,
+                                        task=task, args=args, kwargs=kwargs,
+                                        exc_info=exc_info, worker=self)
+            raise
         except Exception:
             logger.error('Task raised an exception')
             exc_info = sys.exc_info()
@@ -234,7 +265,7 @@ class Worker(object):
             hb.stop()
 
     def _on_heartbeat_error(self, exc_info):
-        self._heartbeat_exc_info = sys.exc_info()
+        self._heartbeat_exc_info = exc_info
         os.kill(os.getpid(), signal.SIGHUP)
 
     @staticmethod
@@ -245,67 +276,59 @@ class Worker(object):
         if kwargs is None:
             kwargs = {}
 
-        start = time.time()
+        start = monotonic()
         try:
             return task.apply(*args, **kwargs)
         finally:
-            end = time.time()
-            logger.info("%s finished in %i seconds." % (task.name, end - start))
+            delta = monotonic() - start
+            logger.info("%s finished in %i seconds." % (task.name, delta))
 
     def _send_reply(self, reply_to, channel, result, exc_info):
         logger.debug("Sending reply result=%r", result)
-        msg = {
-            'result': result,
-        }
+
+        reply = {'result': result}
         if exc_info:
-            type_, val, tb = exc_info
-            msg['exception'] = {
-                'type': '%s.%s' % (type_.__module__, str(type_.__name__)),
-                'value': str(val),
-                'traceback': ''.join(traceback.format_tb(tb)),
-            }
-        msg = amqp.Message(body=json.dumps(msg))
+            reply['exception'] = self._exc_info_dict(exc_info)
+
+        try:
+            body = json.dumps(reply)
+        except Exception as e:
+            logger.error('Cannot serialize result as JSON: %s', e)
+            exc_info = sys.exc_info()
+            reply = {'result': None,
+                     'exception': self._exc_info_dict(exc_info)}
+            body = json.dumps(reply)
+
+        msg = amqp.Message(body=body)
         channel.basic_publish(msg, exchange="", routing_key=reply_to)
 
+    @staticmethod
+    def _exc_info_dict(exc_info):
+        type_, val, tb = exc_info
+        return {
+            'type': '%s.%s' % (type_.__module__, str(type_.__name__)),
+            'value': str(val),
+            'traceback': ''.join(traceback.format_tb(tb))}
+
     def _watch_load(self):
         """Pause consuming messages if lood goes above the allowed limit."""
-        while not self.shutdown_pending.is_set():
-            load = os.getloadavg()[0]
-            if load > self.config.WORKER_MAX_LOAD:
-                if self._pause is False:
-                    logger.warning(
-                        'Load is above the treshold (%.2f/%s), '
-                        'pausing consumer', load, self.config.WORKER_MAX_LOAD)
-                    self._pause = True
-            else:
-                if self._pause is True:
-                    logger.warning(
-                        'Load is below the treshold (%.2f/%s), '
-                        'resuming consumer', load, self.config.WORKER_MAX_LOAD)
-                    self._pause = False
-            time.sleep(1)
+        while not self.shutdown_pending.wait(1):
+            self._current_load = os.getloadavg()[0]
 
     @property
     def uptime(self):
-        if self._started is not None:
-            return os.times()[4] - self._started
+        if self._started_at is not None:
+            return os.times()[4] - self._started_at
 
     def _shutdown_timer(self):
         """Counts down from MAX_WORKER_RUN_TIME. When it reaches zero sutdown
         gracefully.
 
         """
-        if not self.config.WORKER_MAX_RUN_TIME:
-            return
-
-        while not self.shutdown_pending.is_set():
-            remaining = self.config.WORKER_MAX_RUN_TIME - self.uptime
-            if remaining > 0:
-                time.sleep(remaining)
-            else:
-                logger.warning('Run time reached zero')
-                self.shutdown()
-                break
+        remaining = self._max_run_time - self.uptime
+        if not self.shutdown_pending.wait(remaining):
+            logger.warning('Run time reached zero')
+            self.shutdown()
 
     def shutdown(self):
         """Exits after the current task is finished."""
@@ -328,7 +351,7 @@ class Worker(object):
 
         """
         logger.warning("Catched SIGHUP")
-        raise ConnectionError(self._heartbeat_exc_info)
+        raise HeartbeatError(self._heartbeat_exc_info)
 
     @staticmethod
     def _handle_sigusr1(signum, frame):
